/*======================================================================
 * 
 * Программа прошивки зонда на базе Wemos D1R2
 * 
 * ВПИМАНИЕ!!!
 * При изменении версии прошивки не забыть изменить её в макросе ROSA_VER
 * 
 * 2023-01-17 
 * Изменение адреса IP сервера БД
 * 
 * 2021-12-10 
 * Доработка Полины Голуновой:
 * Добавлена настройка подключения к сети Wi-Fi через Web-клиент
 * 
 * 2019-07-17 v 2019-07-17
 * Подключены библиотеки динамического управления датчиками
 * освещённости и цветности
 *
 * 2019-03-27 v 2019-03-27
 * Добавлен поминутный вывод данных в БД dbRobo
 * 
 * 2018-08-02 v 2018-08-02 
 * Уточнено имя зонда для комнатных наблюдений: "Роса-К1"
 * Изменена система нумерации версий 
 * Изменён принцип отображения полей неподключённых датчиков:
 *  теперь всегда показываются все поля всех датчиков,
 *  просто отсутствующие измерения помечаются "none"
 * 
 * 2018-07-27 v _2
 * Ликвидировано зависание программы при отсутствии датчиков
 * Определяется наличие/отсутствие датчиков BH1750, BME280, TCS34725
 * Добавлено отображение уровня сигнала Wi-Fi и собственного адреса MAC
 * Добавлено отображение версии прошивки
 * Точка входа /status заменена на /json
 * По всем остальным адресам отдаётся технический экран
 * Перегруппированы параметры в формате JSON
 * Наличие/отсутствие АЦП ADS1115 определяется 
 *    по наличию/отсутствию его адреса 0x48 на шине I2C
 * Определяется наличие/отсутствие датчика температуры на 1-Wire
 * 
 * 2018-07-25 v _1
 * Зонд получил имя собственное "Роса-1"
 * Откорректировано вычисление мерцания
 * Оставлен вывод собственно параметра мерцания вместо min/max
 * Изменён тип параметра soilH с uint на int (знаковый)
 * Убран вывод значения lux, рассчитанного по rgb
 * Добавлено измерение 1/2 напряжение питания (АЦП, вход 1)
 * Добавлена точка считывания данных /test (возвращает просто Ok!)
 * 
 * 2018-07-18
 * Убран сдвиг влево на 6 значения датчика влажности почвы
 * Дополнительно выведено значение clear в показаниях датчика RGB
 * Убран вывод мощности источника света в показаниях люксометра
 * 
 * 2021-12-10
 * Добавлено подключение к сети через web-страницу (создание собсвтенной точки доступа, если подключение отсутствует)
 * Прибор проверяет подключение к сети каждые 5 минут
 * 
 * Rosa - 192.168.4.1
 * 
 * Столкнулась с проблемой загрузки скетча
 * Решение: в менеджере плат установить версию esp8266 2.5.0. С другими версиями возникают проблемы при загрузке
 * Саму плату выбрать: LOLIN(WEMOS) D1 R2 & mini. Скорость загрузки 115200!! 
 * 
 * Файлы для web-страницы в WeMos загружаются отдельно от скетча: ESP8266 Sketch Data Update
 * Загружаются долго, если происходит сбой при загрузке, следует проверить флеш память и установить: 4М(3M SPIFFS)
 * 
 * После первой загрузки иногда вылетает ошибка при перезагрузке прибора через web-страницу 
 * (проблему пока решить не удалось, возможно какая-то из библиотек конфликтует)
 * В таком случае просто физически перезапустить прибор (отключить-включить питание)
 * 
 * Добавлено определение наличия lcd-дисплея для универсальности скечта
 * Теперь скетч подходит и для Росы-К1, и для Росы-К2
 * 
 * 
 *======================================================================*/



#include <ESP8266WiFi.h> //библиотека для взаимодействия платы с wi-fi сетями
#include <ESP8266WebServer.h> //библиотека для создания веб-сервера
#include <ESP8266HTTPClient.h> 
#include <Wire.h> //библиотека для работы с шиной I2C
#include <LiquidCrystal_I2C.h> //библиотека для работы с LCD 20x4
#include <DNSServer.h>
#include <WiFiManager.h>         
#include <ESP8266SSDP.h>
#include <aREST.h>
#include <FS.h>
#include <ArduinoJson.h> //библиотека для обработки данных
#include <ESP8266WiFiMulti.h> //библиотека для работы с множеством точек досутпа
#include <Ticker.h>
#include <DallasTemperature.h>
#include <Adafruit_TCS34725.h>
#include <BH1750-HD.h> 
#include <BlueDot_BME280.h>
#include <Adafruit_ADS1015.h>
#include <OneWire.h>

#include "tcs34725autorange_lib.h"

#define D1 5
#define D2 4

String ROSA_VER;
String Akey;

Adafruit_ADS1115 ads1115;


int var = 1;
unsigned long when = millis();
unsigned int chet_1=0;
unsigned int chet_2=0;

LiquidCrystal_I2C lcd(0x27, 20, 2);
Ticker deepSleep;

const int  n=20;


ESP8266WiFiMulti wifi;

//Adafruit_ADS1115 ads1115;

#define ONE_WIRE_BUS 2                            // Номер пина датчика 1-Wire
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature TempP(&oneWire);

//Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_700MS, TCS34725_GAIN_1X);
tcs34725 rgb_sensor;

BH1750 lightMeter(0x23);                           // объявляем  переменную для работы с цифровым датчиком освещенности
float LUMf;
uint16_t          LUM_CUR, LUM_MIN, LUM_MAX;       // объявляем  переменные показаний датчика в Люксах, за время измерений 0,5 секунд (текущее, минимальное и максимальное)
uint8_t          LUM_W, LUM_BLINC;                 // объявляем  переменные рассчитываемых показаний (мощность лампы накаливания и коэффициент мерцания)
uint32_t          SEC_START = 0;                   // количество микросекунд на начало измерений

BlueDot_BME280    bme2;

int bh1750Detected = 1;
int bme2Detected = 0;
int tcsDetected = 0;
int ads1115Detected = 0;
int tempDetected = 0;


ESP8266WiFiMulti wifiMulti;
String myAP;     //режим точки доступа
String mySSID;   //флаг использования собственной точки доступа WiFi
IPAddress myIP;  //хранит IP адресс  
String vers="К1";

const int wf=7; //количество сканирируемых сетей
const int wf_configs=20; //количество сохраняемых элементов в памяти
int count_wifi=20;
String ssid[wf_configs]={}; //массив имен сетей
String password[wf_configs]={}; //массив паролей сетей

int k=0;
byte error, address;
bool lcd_q = false;

String ssid_scan[wf]={};

int rssi[wf]={}; //может принимать значения от 0 до -100 дБм (чем выше значение RSSI (ближе к 0), тем сигнал лучше (мощнее))

size_t indexAppendSsid = 0;
const char* _ssid[wf_configs] ={};
const char* _password[wf_configs] ={};


ESP8266WebServer HTTP(80);   //Web интерфейс для устройства
WiFiClient client;
HTTPClient http;
WiFiServer SERVERaREST(8080);   //aREST и сервер для него
aREST rest = aREST();
File fsUploadFile;


String jsonConfig = "{}";                  //переменная, хранящая файл конфигурации

String ssidAP = "Rosa-K1";     // SSID точки доступа
String passwordAP = "";     // Пароль точки доступа

void funDeepSleep()
{
  // Deep Sleep on 0.1s
  ESP.deepSleep(1e5);
}


/////////////////////////////////////////////////////////////////Проверки подключений//
  void setup() 
{
  //lcd.begin(D2,D1);
  lcd.begin();
  lcd.backlight();
  lcd.home();
  // Open connection to Arduino/Iskra
  Serial.begin(115200);
  delay(1500);

  //настраиваем HTTP интерфейс
  HTTP_init();
   //запускаем SSDP сервис
  SSDP_init();

  // регистрируем в aRest функции irControl
  rest.function("irc", irControl);

  // включаем aREST и сервер к нему
  AREST_init();

  //загрузка конфигурации
  LoadConfig(); 
  delay(1000);

 for(size_t i = 0; i < wf_configs; i++) {
      if (ssid[i] != "") 
      {
        count_wifi=count_wifi-1;
      }
  }

  WiFi.persistent(false);
  WiFi.mode(WIFI_STA);  //переключение моудля в режим станции
  
  for (int i=0; i<wf_configs;i++)
  {
     wifiMulti.addAP(_ssid[i],_password[i]);
  }

  ReinitWiFi(0);
  
  delay(10);

  ///////////////////////////////////////////////////// попытаемся определить подключение ADS1115//
  // ADDR->GND  => 0x48 (по умолчанию)
  // ADDR->5V => 0x49
  // ADDR->SDA  => 0x4A
  // ADDR->SCL  => 0x4B
  Wire.begin();
  Wire.beginTransmission(0x48);                     // Проверяем наличие адреса ADS1115 на шине
  byte error = Wire.endTransmission();
  if (error == 0) {
    Serial.println(F("ADS1115 ADC detected!"));
    ads1115Detected = 1;
  } else {
    Serial.println(F("ADS1115 ADC not found!"));
    ads1115Detected = 0;
  }
  
///////////////////////////////////////////////////////Проверка датчика температуры почвы//
  TempP.begin();
  if (TempP.getDeviceCount() > 0) {
    Serial.println(F("1-Wire Sensor detected!"));
    tempDetected = 1;
  } else {
    Serial.println(F("1-Wire Sensor not found!"));
    tempDetected = 0;
  }

////////////////////////////////////////////////////////Проверка датчика освещенности//
  ads1115.begin();
  lightMeter.setMeasurementTime(BH1750_DEFAULT_MTREG);
/*  if (lightMeter.begin(BH1750_CONTINUOUS_LOW_RES_MODE)) {
    Serial.println(F("BH1750 Sensor detected!"));
    bh1750Detected = 1;
  }
  else {
    Serial.println(F("BH1750 Sensor not found!"));
    bh1750Detected = 0;
  }*/

/////////////////////////////////////////////////////////Проверка датчика BME280//

  bme2.parameter.communication = 0;
  bme2.parameter.I2CAddress = 0x76;
  bme2.parameter.sensorMode = 0b11;
  bme2.parameter.IIRfilter = 0b100;
  bme2.parameter.humidOversampling = 0b101;
  bme2.parameter.tempOversampling = 0b101;
  bme2.parameter.pressOversampling = 0b101;
  bme2.parameter.pressureSeaLevel = 1013.25;
  bme2.parameter.tempOutsideCelsius = 15;

  if (bme2.init() != 0x60) {
    Serial.println(F("Second BME280 Sensor not found!"));
    bme2Detected = 0;
  } else {
    Serial.println(F("Second BME280 Sensor detected!"));
    bme2Detected = 1;
  }

//////////////////////////////////////////////////////////Проверка датчика RGB спектра//
  rgb_sensor.begin();
  //if (tcs.begin()) {
  if (rgb_sensor.isAvailable) {
    Serial.println(F("TCS34725 Sensor detected!"));
    tcsDetected = 1;
  } else {
    Serial.println(F("TCS34725 Sensor not found!"));
    tcsDetected = 0;
  }

 ///////////////////////проверка lcd////////////////////// нужный нам адрес=39 (0x27)
  Serial.println("Scanning...");
    if (address=39){
        Wire.beginTransmission(address);
        error = Wire.endTransmission();
 
        if (error == 0){
            Serial.print("I2C device found at address 0x");
            if (address<16)
                Serial.print("0");
                lcd_q = true;
                vers="K2";
                Akey="Kn00Mddnd";
                ROSA_VER="K2-2023-01-17";
                ssidAP = "Rosa-K2";
            Serial.print(address,HEX);
        }
        else {
                lcd_q = false;
                vers="K1";
                Akey= "RosaSchoolArtek";
                ROSA_VER="K1-2023-01-17";
                Serial.println("NO lcd");
        } 
    }
 
///////////////////////////////////////////////////////////Название сети WIFI и IP адрес//

 // deepSleep.attach(300., funDeepSleep);
}


/////////////////////////////////////////////////////////////Вывод на LCD дисплей//
void loop(void)
{
  if (millis() - when >= 2500)
  {
    when = millis();
    chet_1++;
    chet_2++;
    if (lcd_q){
    switch (var)
    {
      case 1:
        lcd.clear();
                                   lcd.print("T:");        lcd.print(String(bme2.readTempC()) + " C");                      //Температура воздуха
        lcd.setCursor(0, 1);       lcd.print("H:");        lcd.print(String(bme2.readHumidity()) + " %");                   //Влажность воздуха
        var = 2;
        break;

      case 2:
        lcd.clear();
                                   lcd.print("P:");        lcd.print(String(bme2.readPressure() * 0.750064) + " mm Hg");    //Давление
        
        var = 3;
        break;

      case 3:
        lcd.clear();
                                   lcd.print("L:");        lcd.print(String(LUMf) + " Lux");                                //Освещенность
        lcd.setCursor(0, 1);       lcd.print("Blink:");        lcd.print(String(LUM_BLINC));                                //Мерцание
        var = 4;
        break;

      case 4:
        //uint16_t r, g, b, c, colorTemp;
        //tcs.getRawData(&r, &g, &b, &c);
        //colorTemp = tcs.calculateColorTemperature(r, g, b);
  rgb_sensor.getData();
        lcd.clear();
//                                   lcd.print("t:");        lcd.print(String(colorTemp));                                    //Температура цвета
//        lcd.setCursor(9, 0);       lcd.print("R:");        lcd.print(String(r));                                            //красный
//        lcd.setCursor(0, 1);       lcd.print("G:");        lcd.print(String(g));                                            //зеленый
//        lcd.setCursor(9, 1);       lcd.print("B:");        lcd.print(String(b));                                            //синий
                                   lcd.print("t:");        lcd.print(String(rgb_sensor.ct));                                    //Температура цвета
        lcd.setCursor(9, 0);       lcd.print("R:");        lcd.print(String(rgb_sensor.r_comp));                                            //красный
        lcd.setCursor(0, 1);       lcd.print("G:");        lcd.print(String(rgb_sensor.g_comp));                                            //зеленый
        lcd.setCursor(9, 1);       lcd.print("B:");        lcd.print(String(rgb_sensor.b_comp));                                            //синий
        var = 5;
        break;

      case 5:
        lcd.clear();
                                   lcd.print("SoilT:");     lcd.print(TempP.getTempCByIndex(0));   lcd.print(" C");         //Температура почвы
        lcd.setCursor(0, 1);       lcd.print("SoilH:");     lcd.print(String(ads1115.readADC_SingleEnded(0)));              //Влажность почвы
        var = 6;
        break;
        
      case 6:
        lcd.clear();
        lcd.setCursor(0, 0); lcd.print("SSID: "); lcd.print(mySSID);
        lcd.setCursor(0, 1); lcd.print("IP: "); lcd.print(myIP);
        var =1;
    }
    }
  }
  //передача данных в базу раз в минуту + вывод подключенной сети
  if (chet_2 == 22)
  {
      //String status = getStatus();
      Serial.println();
      Serial.print("WiFi SSID:");
      Serial.println(mySSID);
      Serial.print("WiFi IP:");
      Serial.println(myIP);
      Serial.print("Mode:");
      Serial.println(myAP);

// Дополнение для более новой версии библиотек ESP8266
  //WiFiClient client;
  HTTPClient http;
  //http.begin("http://188.35.161.31/core/jsonadd.php");
  //http.begin(client,"http://188.35.161.31/core/jsonadd.php");
  http.begin("http://dbrobo.mgul.ac.ru/core/jsonadd.php");
// Вариант для более новой версии библиотек ESP8266
  //http.begin(client,"http://188.35.161.31/core/jsonadd.php");
  http.addHeader("Content-Type","application/json");
  Serial.println(getStatus());
  http.POST(getStatus());
  //http.GET();
  http.end();
      chet_2=0;
  }
  //проверка/переподключение к сети wi-fi раз в 5 минут
  if(k==0 || chet_1==110)
  {
  ReinitWiFi(1);
  chet_1=0;
  k=1;
  }
  HTTP.handleClient();
  delay(1);
  WiFiClient client = SERVERaREST.available();
  if (!client) 
  {
    return;
  }
  while (!client.available()) 
  {
    delay(1);
  }
  rest.handle(client);
}


String getStatus() {
  String reply = ""; 
  // ADC
  int16_t soilH;
  int16_t Upit;
  char soilT[10];
  char UpitV[10];

  reply = "{ ";

  if(ads1115Detected){
    soilH = ads1115.readADC_SingleEnded(0);
    Upit = ads1115.readADC_SingleEnded(1);
    dtostrf(6.144/32768*Upit*2,6,2,UpitV);
  }
  
  if(tempDetected){
    TempP.requestTemperatures();
    dtostrf(TempP.getTempCByIndex(0),6,2,soilT);
  }
  
  reply += "\"system\":{ ";
  reply += "\"Akey\":\"" + String(Akey) + "\",";
  reply += "\"Version\": \"" + String(ROSA_VER) + "\", ";
  if(ads1115Detected){
    reply += "\"Upit\":" + String(UpitV) + ", ";
  }else{
    reply += "\"Upit\":\"none\", ";
  }
  reply += "\"RSSI\":" + String(WiFi.RSSI()) + ", ";
  reply += "\"MAC\":\"" + String(WiFi.macAddress()) + "\" },";
  
  reply += " \"soil\":{ ";
  if(ads1115Detected){
    reply += "\"soilH\":" + String((soilH)) + ", ";
  }else{
    reply += "\"soilH\":\"none\", ";
  }
  if(tempDetected){
    reply += "\"soilT\":" + String(soilT) ;
  }else{
    reply += "\"soilT\":\"none\"";
  }
  reply += " }, ";

  reply += getColor() + ", " + getLight() + ", ";
 
  if (bme2Detected){
    reply += "\"weather\":{ ";
    reply += "\"temp\":" + String(bme2.readTempC()) + ", ";
    reply += "\"humidity\":" + String(bme2.readHumidity()) + ", ";
    reply += "\"pressure\":" + String(bme2.readPressure()*0.750064) + " }";
  }else{
    reply += "\"weather\":{ ";
    reply += "\"temp\":\"none\", ";
    reply += "\"humidity\":\"none\", ";
    reply += "\"pressure\":\"none\" }";
  }
  
  reply += " } ";
  return (reply);
}

///////////////////////////////////////////////////////////////RGB спектр//
String getColor() {
  String reply = "\"color\":{";
  //uint16_t r, g, b, c, colorTemp, lux;

  if (tcsDetected == 0) {
    reply += "\"tempCT\":\"none\",";
    reply += "\"lux\":\"none\",";
    reply += "\"clearC\":\"none\",";
    reply += "\"ir\":\"none\",";
    reply += "\"redC\":\"none\",";
    reply += "\"greenC\":\"none\",";
    reply += "\"blueC\":\"none\"}";
    return (reply);
  }

  //tcs.getRawData(&r, &g, &b, &c);
  //colorTemp = tcs.calculateColorTemperature(r, g, b);
  //lux = tcs.calculateLux(r, g, b);
  rgb_sensor.getData();
  reply += "\"tempCT\":" + String(rgb_sensor.ct) + ",";
  reply += "\"lux\":" + String(rgb_sensor.lux) + ",";
  reply += "\"clearC\":" + String(rgb_sensor.c_comp) + ",";
  reply += "\"ir\":" + String(rgb_sensor.ir) + ",";
  reply += "\"redC\":" + String(rgb_sensor.r_comp) + ",";
  reply += "\"greenC\":" + String(rgb_sensor.g_comp) + ",";
  reply += "\"blueC\":" + String(rgb_sensor.b_comp) + "}";
  return (reply);
}

//////////////////////////////////////////////////////////////////Освещенность//
String getLight() {
  int i;
  String reply = "\"light\":{";
  LUM_MAX   = 0;                                          // устанавливаем минимальное для максимального значения
  LUM_MIN   = 65535;                                      // устанавливаем максимальное для минимального значения
  LUM_W     = 5;                                          // устанавливаем минимальное значение освещенности в Вт

  if (bh1750Detected == 0) {
    reply += "\"lux\":\"none\",\"blink\":\"none\"}";
    return (reply);
  }

///////////////////////// Включаем режим максимальной частоты опроса
  lightMeter.setMeasurementTime(BH1750_DEFAULT_MTREG);
  lightMeter.begin(BH1750_CONTINUOUS_LOW_RES_MODE);
  delay(20);
  
//////////////////////// затравочное чтение
  for (i = 0; i < 10; i++) {
    LUM_CUR = lightMeter.readLightLevel();
    //LUM_CUR = bh1750.readLin ghtLevel();                  // считываем показания с датчика
    delay(16);                                              // задержка 16 мс (цикл преобразования датчика)
  }
 
//////////////////////// измерение
  for (i = 0; i < 30; i++) {
    LUM_CUR = lightMeter.readLightLevel();
    //LUM_CUR = bh1750.readLightLevel();                    // считываем показания с датчика
    if (LUM_MIN > LUM_CUR) {
      LUM_MIN = LUM_CUR;                                    // обновляем минимальное  значение
    }
    if (LUM_MAX < LUM_CUR) {
      LUM_MAX = LUM_CUR;                                    // обновляем максимальное значение
    }
    delay(16);                                              // задержка 16 мс (цикл преобразования датчика)
  }

///////////////////////Определяем коэффициент мерцания
  // Правильно это называется "Процент мерцания" (Percent Flicker),
  // или по нашему СНиП "Коэффициент пульсации"
  
  if (LUM_MAX == 0 && LUM_MIN == 0) {
    LUM_BLINC = 0;
  }
  else {
    LUM_BLINC = (int)( (LUM_MAX - LUM_MIN) * 100. / (LUM_MAX + LUM_MIN) );
  }
  
//////////////////////Читаем значение освещённости
  // с максимальным разрешением датчика
  
  lightMeter.begin(BH1750_AUTO_MODE);
  delay(445);
  LUMf = lightMeter.readLightLevel();
  delay(445);
  LUMf = lightMeter.readLightLevel();
  delay(445);
  LUMf = lightMeter.readLightLevel();


  reply += "\"lux\":" + String(LUMf) + ",\"blink\":" + String(LUM_BLINC) + "}";
  return (reply);
}
  
  /*

  Зонд \"Роса-К2\"\n\
  Мытищинский филиал МГТУ им. Н. Э. Баумана\n\
  Кафедра прикладной математики, информатики и вычислительной техники (К-3)\n\
  \n\
  Разработчики:\n\
  Малашин Алексей Анатольевич  malashin_a@mail.ru\n\
  Чернышов Александр Викторович sch-ru@yandex.ru\n\
  Виноградов Дмитрий Владимирович dmitry-vinogradov@yandex.ru\n\
  Брюквина Ольга Юрьевна    bryukvina_o@mail.ru\n\
  Тищенко Юлия Александровна  tishenko.yulya@bk.ru\n\
  Курмаев Антон Мартынович    warrior-rinn@yandex.ru\n\
  \n\
  "; */
